#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

import Commands
import QUB

import win32com.client
from pythoncom import CoInitialize, CoUninitialize

QUB_Colors = [0, 0x0000ff, 0x800000, 0x008000, 0x00ffff, 0xff00ff, 0x00ff00, 0xff0000, 0xffff00, 0x800080]

def MakeHistogramSheet(hist, sheet, showComponents=1, w=500, h=280):
    cells = sheet.Cells
    cells(1,1).Value = str(hist['Title'].data)
    cells(2,1).Value = str(hist['XLabel'].data)
    cells(2,2).Value = str(hist['YLabel'].data)

    sheet.ChartObjects().Add(150,50,w,h)
    chart = sheet.ChartObjects(1).Chart
    chart.ChartType = 51 # xlColumnClustered
    while chart.SeriesCollection().Count > 0:
        chart.SeriesCollection(1).Delete

    nbin = hist['BinCount'].data[0]
    bins = hist['Bins']
    bars = hist['Bars']

    for i in range(0,nbin):
        cells(i+3,1).Value = bins.data[i]
        cells(i+3,2).Value = bars.data[i]

    chart.SeriesCollection().Add( sheet.Range( sheet.Cells(2,1), sheet.Cells(nbin+2,2) ), 2, 0, 1, 0)
        
    col = 3

    if showComponents:
        line = hist.find('Component')
        while not line.isNull:
            for i in range(0,nbin):
                cells(i+3,col).Value = line.data[i]

            chart.SeriesCollection().Add( sheet.Range( sheet.Cells(2,col), sheet.Cells(nbin+2,col) ), 2, 0, 0, 0)
            chart.SeriesCollection(col-1).ChartType = 4 # xlLine  # xlLineStacked = 63
            chart.SeriesCollection(col-1).Border.Color = 0x999999
            chart.SeriesCollection(col-1).Border.Weight = 0x2 # xlThin    
            line = line.nextSameName()
            col = col + 1
                
    line = hist.find('Line')
    while not line.isNull:
        for i in range(0,nbin):
            cells(i+3,col).Value = line.data[i]

        try:
            linecolor = line.find('Color').data[0]
        except:
            linecolor = 0

        chart.SeriesCollection().Add( sheet.Range( sheet.Cells(2,col), sheet.Cells(nbin+2,col) ), 2, 0, 0, 0)
        chart.SeriesCollection(col-1).ChartType = 4 # xlLine  # xlLineStacked = 63
        chart.SeriesCollection(col-1).Border.Color = QUB_Colors[ linecolor ]
        chart.SeriesCollection(col-1).Border.Weight = -4138 # 0xffffefd6 # xlMedium    
        line = line.nextSameName()
        col = col + 1
            
    yaxis = chart.Axes(2)
    yaxis.HasTitle = 1
    yaxis.AxisTitle.Text = str(hist['YLabel'].data)
    if yaxis.MinimumScale < 0.0:
        yaxis.CrossesAt = yaxis.MinimumScale

    chart.Axes(1).HasTitle = 1
    # chart.Axes(1).AxisTitle.Font.Size = 6
    chart.Axes(1).AxisTitle.Caption = str(hist['XLabel'].data)

    chart.HasLegend = 0
    chart.PlotArea.Interior.ColorIndex = 0
    chart.HasTitle = 1
    chart.ChartTitle.Text = str(hist['Title'].data)

    return sheet.ChartObjects(1)    

def ExtractToExcel(hist, showComponents=1):
    #CoInitialize() -- let WordReport.py do this on startup

    try:
        win32com.client.gencache.EnsureModule('{00020813-0000-0000-C000-000000000046}',0,1,3)
    except:
        pass

    try:
        excel = win32com.client.Dispatch("Excel.Application")
        book = excel.Workbooks.Add(-4167) # XLWBatWorksheet
        sheet = book.Worksheets.Item(1)
        excel.Visible = 1

        MakeHistogramSheet(hist, sheet, showComponents)
    finally:
        pass
        #CoUninitialize()

Commands.Register('Histogram', 'Extract to Excel', ExtractToExcel)
